@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2007 Zuza Software Foundation
 #
@@ -19,9 +18,10 @@
 
 """This module represents the Vietnamese language.
 
-.. seealso:: http://en.wikipedia.org/wiki/Vietnamese_language
+.. seealso:: :wp:`Vietnamese_language`
 """
 
+
 from translate.lang import common, fr
 
 
@@ -31,8 +31,8 @@ class vi(common.Common):
     # Vietnamese uses similar rules for spacing two-part punctuation marks as
     # French, but does not use a space before '?'.
     puncdict = {}
-    for c in u":;!#":
-        puncdict[c] = u" %s" % c
+    for c in ":;!#":
+        puncdict[c] = " %s" % c
 
     @classmethod
     def punctranslate(cls, text):
@@ -42,7 +42,7 @@ class vi(common.Common):
             - % and $ are not touched yet for fear of variables
             - Double spaces might be introduced
         """
-        text = super(cls, cls).punctranslate(text)
+        text = super().punctranslate(text)
         return fr.guillemets(text)
 
     mozilla_nplurals = 2
